@@ -1,8 +1,7 @@
 <?php
 /* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
-/* @(#) $Header: /sources/phpprintipp/phpprintipp/CupsPrintIPP.php,v 1.10 2006/01/20 19:49:39 harding Exp $
+/* @(#) $Header: /sources/phpprintipp/phpprintipp/CupsPrintIPP.php,v 1.12 2008/05/12 05:14:15 harding Exp $
  *
- * Version: 0.2
  * Class PrintIPP - Send extended IPP requests.
  *
  *   Copyright (C) 2005-2006  Thomas HARDING
@@ -491,10 +490,12 @@ class CupsPrintIPP extends ExtendedPrintIPP {
       
         $this->available_printers = array();
         $k = 0;
-        
+        $this->printers_attributes = new stdClass();
+
         for ($i = 0 ; (array_key_exists($i,$this->serveroutput->response)) ; $i ++)
             if (($this->serveroutput->response[$i]['attributes']) == "printer-attributes") {
                 $phpname = "_printer".$k;
+                $this->printers_attributes->$phpname = new stdClass();
                 for ($j = 0 ; array_key_exists($j,$this->serveroutput->response[$i]) ; $j++) {
                 
                     $value = $this->serveroutput->response[$i][$j]['value'];
@@ -506,6 +507,7 @@ class CupsPrintIPP extends ExtendedPrintIPP {
                             break;
                         case "printer_type":
                             $table = self::_interpretPrinterType($value);
+                            $this->printers_attributes->$phpname->$name = new stdClass();
         
                             for($l = 0 ; $l < count($table) ; $l++ ) {
                                 $index = '_value'.$l;
