@@ -41,16 +41,17 @@
 *
 ***********************************************************/
 
-$host = "geekette.localdomain"; // set serve'rs host here
-$ssl = true; // enable ssl if true
-$printer_uri = "/printers/epson"; // set printer uri here
+$host = "localhost"; // set serve'rs host here
+$ssl = false; // enable ssl if true
+$printer_uri = "/printers/Parallel_Port_1"; // set printer uri here
 $language = "fr_fr";
 $user = "test"; // valid user from lpadmin group
 $password = "0123Test"; // his password
 $document_uri = "http://localhost/";
-$debug_level = 1; // 1: silent; 3: very verbose
+$debug_level = 3; // 1: silent; 3: very verbose
 
-error_reporting(E_ALL);
+
+error_reporting(E_ALL|E_STRICT);
 /******************
 *
 * END OF SETUP
@@ -113,8 +114,7 @@ $j = 0;
 $test = 1;
 
 /* printing a large file */
-/*
-echo "<br /><br /><br />TEST ".$test++."<br />";
+echo "<br /><br /><br />TEST 1<br />";
 echo "OPERATION  ".$j++."<br />";
 $ipp->setAttribute("printer-resolution","1440x720dpi");
 $ipp->setAttribute("job-billing", "Thomas");
@@ -131,8 +131,7 @@ $ipp->setAttribute("scaling","");
 
 
 /* getting printer's attributes */
-/*
-echo "<br /><br /><br />TEST ".$test++."<br />";
+echo "<br /><br /><br />TEST 2<br />";
 echo "OPERATION  ".$j++."<br />";
 echo "Getting Printer's attributes: ". $ipp->getPrinterAttributes() . "<br />";
 printf('$ipp->status[%s] = %s <br /><br />', count($ipp->status) -1,$ipp->status[count($ipp->status) -1]);
@@ -141,8 +140,7 @@ echo "Printer up-time: ".date('Y-m-d H:i:s',$timestamp)."<br />";
 echo "Printer's attributes :<pre>\n"; print_r($ipp->printer_attributes); echo "</pre>";
 
 /* printing a string */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 3<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 $ipp->setCopies(2);
@@ -155,8 +153,7 @@ echo "Job Attributes:<pre>\n" ; print_r($ipp->job_attributes) ; echo "\n</pre>\n
 $first_job = $ipp->last_job;
 
 /* printing a text file */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 4<br />";
 
         // HINT: you _must_ supply a charset or set output as raw text
         // note that mimeMediaType is resetted to octet-stream after each call of printJob (this is a feature).
@@ -166,14 +163,13 @@ $ipp->setMimeMediaType('text/plain');
 $ipp->setJobName("PHP Test: US ASCII file",true); // default is false: number is automagically appended
 $ipp->setData("./testfiles/test.txt");//Path to file.
 $ipp->setAttribute("job-sheets", array("confidential"));
-echo "US ASCII file Job status: ".$ipp->printJob("epson")."<br />";
+echo "US ASCII file Job status: ".$ipp->printJob()."<br />";
 $ipp->setAttribute("job-sheets","");
 $ipp->setAttribute("cpi",""); //reset cpi
 $ipp->setAttribute("lpi","");
 
 /* printing a document by URI */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 5<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 echo "Job Print URI status: ".$ipp->printUri($document_uri)."<br />";
@@ -182,7 +178,7 @@ echo "Job Print URI status: ".$ipp->printUri($document_uri)."<br />";
 
 /* printing a text file in utf-8 */
 /*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 6<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 $ipp->setUserName("foo bar");
@@ -199,8 +195,7 @@ echo "Printing testfile with UTF-8 characters: ".$ipp->printJob()."<br />";
 $ipp->unsetAttribute('orientation-requested');
 
 /* printing a gzipped file */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 7<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 $ipp->setAttribute('compression','gzip');
@@ -210,8 +205,7 @@ $ipp->unsetAttribute('compression');
 
 
 /* getting not-completed jobs */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 8<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 echo "Getting NOT PROCESSED Jobs: ".$ipp->getJobs(true,0,"")."<br />"; // defaults to $my_jobs=true,$limit=0 (no limit),$which_jobs=not-completed,$subset=false 
@@ -221,11 +215,11 @@ echo "Job 0 state-reasons: ".$ipp->jobs_attributes->job_0->job_state_reasons->_v
 echo "<pre>";print_r($ipp->jobs_attributes); echo "</pre>";
 
 /* getting all jobs (subset of attributes) */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 9<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
-echo "Getting ALL Jobs: ".$ipp->getJobs(true,0,"completed",true)."<br />";
+#echo "Getting ALL Jobs: ".$ipp->getJobs(true,0,"completed",true)."<br />";
+echo "Getting ALL Jobs: ".$ipp->getJobs(true)."<br />";
 
 echo "Job 0 state: ".$ipp->jobs_attributes->job_0->job_state->_value0."<br />";
 echo "Job 0 state-reasons: ".$ipp->jobs_attributes->job_0->job_state_reasons->_value0."<br />";
@@ -233,9 +227,14 @@ echo "<pre>";print_r($ipp->jobs_attributes); echo "</pre>";
 
 
 /* getting a job's attributes */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 10<br />";
 
+echo "OPERATION ". $j ++ ."<br />";
+echo "print a job";
+$ipp->setData("test");
+$ipp->printJob();
+$job = $ipp->last_job;
+echo "Job uri: ",$job,"<br />\n";
 echo "OPERATION ". $j ++ ."<br />";
 echo "Getting last job's attributes: ".$ipp->getJobAttributes($ipp->last_job,false,'all')."<br />";
                     
@@ -252,31 +251,34 @@ echo "Job-State-Reasons: $job_state_reasons<br />";
  
 echo "<pre>";print_r($ipp->job_attributes); echo "</pre>";
 
+echo "OPERATION ". $j ++ ."<br />";
+echo "cancelling the job";
+$ipp->cancelJob($job);
+
 /* setting job's attributes to new values */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 11<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
-echo "deleting print-quality attribute<br />";
-$unset_attributes = array("print-quality");//,"printer-resolution");
-echo "setting copies to 2<br />";
+echo "Printing a file <br />";
+$ipp->setData("testfiles/test.ps");
+echo $ipp->printJob();
+$job = $ipp->last_job;
+echo "OPERATION ". $j ++ ."<br />";
+echo "modify copies nbr to 2<br />";
 $ipp->setCopies(2);
-echo "setting resolution to 320x200dpi<br />";
-$ipp->setAttribute('printer-resolution','320x200dpi');
-echo "Setting last job's attributes: ".$ipp->setJobAttributes($ipp->last_job,$unset_attributes)."<br />";
+echo "Setting last job's attributes: ".$ipp->setJobAttributes($job)."<br />";
 echo "OPERATION ". $j ++ ."<br />";
 echo "Getting job's attributes: ".$ipp->getJobAttributes($ipp->last_job)."<br />";
-$resolution = $ipp->job_attributes->printer_resolution->_value0;
-echo "Resolution: $resolution<br />";
-$print_quality = $ipp->job_attributes->print_quality->_value0;
-echo "Print-Quality: $print_quality<br />";
+$copies = $ipp->job_attributes->copies->_value0;
+echo "Copies: $copies<br />";
  
 echo "<pre>";print_r($ipp->job_attributes); echo "</pre>";
-
+echo "OPERATION ". $j ++ ."<br />";
+echo "deleting job<br />";
+$ipp->cancelJob($job);
 
 /* printing selected pages from a document */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 12<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 $ipp->setAttribute('media','A7');
@@ -291,25 +293,24 @@ $ipp->unsetAttribute('number-up');
 
 /* printing a postcript file */
 /*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 13<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 $ipp->setMimeMediaType(); // => autodetection
 $ipp->setData("./testfiles/test.ps");
-echo "Printing Postscript Job status: ".$ipp->printJob("epson")."<br />";
+echo "Printing Postscript Job status: ".$ipp->printJob()."<br />";
 
 
 /* printing a png  file */
 /*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 14<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 $ipp->setData("./testfiles/test.png");
-echo "Printing png Job status: ".$ipp->printJob("epson")."<br />";
+echo "Printing png Job status: ".$ipp->printJob()."<br />";
 
 /* printing and cancelling job */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 15<br />";
 
 echo "Printing and Cancelling a job<br />";
 $ipp->setData("./testfiles/test.ps");
@@ -321,8 +322,7 @@ echo "OPERATION ". $j ++ ."<br />";
 echo "Cancel status: ".$ipp->cancelJob($job)."<br />";
 
 /* printing strings, no form feed */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 16<br />";
 
 $ipp->setRawText();
 $ipp->unsetFormFeed();
@@ -331,35 +331,34 @@ echo "Printing RAW TEXT strings<br />";
 
 $ipp->setData("This is a line\n");
 echo "OPERATION ". $j ++ ."<br />";
-echo "Job status: ".$ipp->printJob("epson")."<br />";
+echo "Job status: ".$ipp->printJob()."<br />";
 
 $ipp->setData("This is half a line ");
 echo "OPERATION ". $j ++ ."<br />";
-echo "Job status: ".$ipp->printJob("epson")."<br />";
+echo "Job status: ".$ipp->printJob()."<br />";
 
 $ipp->setData("This is a end of line\n");
 echo "OPERATION ". $j ++ ."<br />";
-echo "Job status: ".$ipp->printJob("epson")."<br />";
+echo "Job status: ".$ipp->printJob()."<br />";
 
 // set copies to 2 (same sheet of paper: form feed is unset)
 $ipp->setData("This lines must appeared twice\r\n");
 $ipp->setCopies(2);
 echo "OPERATION ". $j ++ ."<br />";
-echo "Job status: ".$ipp->printJob("epson")."<br />";
+echo "Job status: ".$ipp->printJob()."<br />";
 $ipp->setCopies(1);
 
 // printing string, then form feed
 echo "OPERATION ". $j ++ ."<br />";
 $ipp->setFormFeed();
 $ipp->setData("End of test");
-echo "Job status: ".$ipp->printJob("epson")."<br />";
+echo "Job status: ".$ipp->printJob()."<br />";
 
 $ipp->unsetRawText();
 
 
 /* printing a file to see if unsetRawText works */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 17<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 $ipp->setData("./testfiles/test.ps");
@@ -367,8 +366,7 @@ echo "Job status (text file after strings): ".$ipp->printJob()."<br />";
 
 
 /* multiple document handling */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 18<br />";
 
 $ipp->setUserName("test");
 $ipp->setCopies(1);
@@ -397,8 +395,7 @@ echo "Sending document (must be refused): " . $ipp->sendDocument($job,$last=true
 
 
 /* try to validate a job with filetype printer server can't handle */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 19<br />";
 
 $ipp->setMimeMediaType("application/x-foobar");
 
@@ -406,7 +403,7 @@ echo "OPERATION ". $j ++ ."<br />";
 echo "Validate-Job for a document with file format 'application/x-foobar': ".$ipp->validateJob()."<br />";
 
 $ipp->setMimeMediaType();
-
+echo "\n<br />MEDIATYPE ",$ipp->meta->mime_media_type,"<br />\n";
 foreach ($ipp->attributes as $name => $attribute) {
     if ($attribute->_range == 'unsupported-attributes')
         printf('%s "%s": unsupported attribute<br />',$name,$attribute->_value0);
@@ -416,8 +413,7 @@ echo "Details:<pre>"; print_r($ipp->attributes) ; echo "</pre>";
 
 
 /* Printing then holding a job */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 20<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 echo "Printing a document :".$ipp->printJob()."<br />";
@@ -431,8 +427,7 @@ echo "Job State: ".$ipp->job_attributes->job_state->_value0."<br />";
 echo "Job State Reason: ".$ipp->job_attributes->job_state_reasons->_value0."<br />";
 
 /* releasing the job */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 21<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 echo "Releasing the job: ".$ipp->releaseJob($job,'indefinite')."<br />";
@@ -443,8 +438,7 @@ echo "Job State Reason: ".$ipp->job_attributes->job_state_reasons->_value0."<br
 
 
 /* restarting a job */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 22<br />";
 echo "OPERATION ". $j ++ ."<br />";
 echo "Restarting the job $first_job (if completed!): ".$ipp->restartJob($first_job)."<br />";
 echo "OPERATION ". $j ++ ."<br />";
@@ -454,17 +448,15 @@ echo "Job State Reason: ".$ipp->job_attributes->job_state_reasons->_value0."<br
 
 
 /* purging jobs for a printer */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 23<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 $ipp->setUserName("test");
-$ipp->setPrinterURI("ipp://localhost:631/printers/epson"); // Set printer URI here
+$ipp->setPrinterURI($printer_uri); // Set printer URI here
 echo "Purge-Jobs for printer $printer_uri: ". $ipp->purgeJobs() ."<br />";
 
 /* purging jobs for all printers */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 24<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 $ipp->setPrinterURI("ipp://localhost:631/printers/"); // => all printers
@@ -472,8 +464,7 @@ echo "Purge-Jobs (all printers): ". $ipp->purgeJobs() ."<br />";
 
 
 /* pausing printer */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 25<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 $ipp->setPrinterURI($printer_uri);
@@ -484,8 +475,7 @@ echo "Getting printer's attributes: ".$ipp->getPrinterAttributes()."<br />";
 echo "Printer State: ".$ipp->printer_attributes->printer_state->_value0."<br />";
 
 /* Resuming printer */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 26<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 echo "Resuming Printer $printer_uri, then sleep 1 second: ".$ipp->resumePrinter()."<br />";
@@ -495,7 +485,7 @@ echo "Getting printer's attributes: ".$ipp->getPrinterAttributes()."<br />";
 echo "Printer State: ".$ipp->printer_attributes->printer_state->_value0."<br />";
 
 /* setting printer's attributes */
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 27<br />";
 
 echo "OPERATION ". $j ++ ."<br />";
 
@@ -513,13 +503,11 @@ echo "</pre>\n";
 
 
 /* rejecting jobs (CUPS specific)*/
-/*
-echo "<br /><br /><br />TEST ".$test++."<br />";
+echo "<br /><br /><br />TEST 28<br />";
 echo "OPERATION  ".$j++."<br />";
 echo "Rejecting jobs for printer $printer_uri (CUPS operation): ".$ipp->cupsRejectJobs($printer_uri,$printer_state_message="Printer stopped for maintainance")."<br />";
 
 // try to print //
-/*
 echo "OPERATION ". $j ++ ."<br />";
 $ipp->setCharset('us-ascii');
 $ipp->setMimeMediaType('text/plain');
@@ -528,7 +516,7 @@ $ipp->setJobName("PHP Test: US ASCII file",true); // default is false: number is
 $ipp->setAttribute("cpi",17); // 10,12 or 17. default to 10 //cpi and lpi attributes are CUPS specific
 $ipp->setAttribute("lpi",8); // 6 or 8, default: 6
 $ipp->setData("./testfiles/test.txt");//Path to file.
-echo "US ASCII file Job status: ".$ipp->printJob("epson")."<br />";
+echo "US ASCII file Job status: ".$ipp->printJob()."<br />";
 $ipp->setAttribute("cpi",""); //reset cpi
 $ipp->setAttribute("lpi","");
 
@@ -542,14 +530,12 @@ echo "OPERATION  ".$j++."<br />";
 echo "Accepting jobs for printer $printer_uri (CUPS operation): ".$ipp->cupsAcceptJobs($printer_uri)."<br />";
 
 /* printing the text file */
-/*
 echo "OPERATION  ".$j++."<br />";
-echo "US ASCII file Job status: ".$ipp->printJob("epson")."<br />";
+echo "US ASCII file Job status: ".$ipp->printJob()."<br />";
 
 
 /* getting only default printer's uri (CUPS extention) */
-/*
-echo "<br /><br /><br />TEST ".$test++."<br />";
+echo "<br /><br /><br />TEST 29<br />";
 echo "OPERATION ".$j ++."<br />";
 
 printf("Default printer URI [CUPS specific]: %s<pre>\n",$ipp->cupsGetDefaults(array('printer-uri-supported')));
@@ -559,14 +545,15 @@ print_r($ipp->printer_attributes); echo "</pre>";
 
 
 /* getting default printer attributes (CUPS extension) */
-/*
-echo "<br /><br /><br />TEST ".$test++."<br />";
+echo "<br /><br /><br />TEST 30<br />";
 echo "OPERATION ".$j ++."<br />";
 echo "Getting default printer attributes [CUPS specific]<br /><pre>\n";
+$ipp->setAuthentication($user,$password);
 $ipp->cupsGetDefaults(array('all'));
 
 print_r($ipp->printer_attributes);
 
+/* No longer supported?
 echo "</pre>\nPrinter historic for default printer:\n"; 
     
 $histo = $ipp->printer_attributes->printer_state_history->_value1;
@@ -585,11 +572,10 @@ $histo = $ipp->printer_attributes->printer_state_history->_value1;
     $idx_histo = '_indice'. ($idx + 1);
     }
 echo "<br /><br /><br />\n";
-
+*/
 
 /* getting printers (vendor extention) (currently CUPS only)*/
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 31<br />";
 
 echo "OPERATION ".$j++."<br />";
 echo "Available printers [CUPS EXTENTION]:<pre>" ; 
@@ -629,8 +615,7 @@ echo "------- END FOR PRINTER $i -------------<br /n>";
 
 
 /* send a buggy request */
-/*
-echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "<br /><br /><br /> TEST 32<br />";
 echo "OPERATION ". $j ++ ."<br />";
 $ipp->setData("This is an error : nothing printed\n");
 $ipp->generateError ("request_body_malformed");
