@@ -1,11 +1,10 @@
 <?php
 /* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
- /* @(#) $Header: /sources/phpprintipp/phpprintipp/http_class.php,v 1.5 2006/04/30 14:14:33 harding Exp $
+ /* @(#) $Header: /sources/phpprintipp/phpprintipp/http_class.php,v 1.8 2008/05/12 05:14:15 harding Exp $
  *
  *
- * Version: 0.2 -- E X P E R I M E N T A L
- *
  * Class http_class - Basic http client with "Basic" authorization mechanism.
+ * handle ipv6 addresses and https
  *
  *   Copyright (C) 2006  Thomas HARDING
  *
@@ -46,6 +45,40 @@
     TODO: beta tests on servers other than loopback one
 */
 
+/***********************
+*
+* httpException class
+*
+************************/
+
+    // {{{ class httpException
+class httpException extends Exception {
+    
+    protected $errno;
+
+    public function __construct($msg,$errno=null) {
+        parent :: __construct($msg);
+        $this->errno = $errno;
+    }
+
+    public function getErrorFormatted() {
+    
+        $return = sprintf("[http_class]: %s -- "._(" file %s, line %s"),
+                $this->getMessage(),
+                $this->getFile(),
+                $this->getLine());
+
+    return $return;
+    }
+
+    public function getErrno() {
+    
+    return $this->errno ;
+    }
+
+}
+    // }}}
+
 /*************************
 *
 * class http_class
@@ -66,9 +99,9 @@ class http_class {
     public $request_body = "Not a useful information";
     public $status;
     public $window_size = 1400; // chunk size of data
-    
+    public $with_exceptions = 0; // compatibility mode for old scripts
+
     private $default_port = 631;
-    
     private $headers;
     private $reply_headers = array();
     private $reply_body = array();
@@ -117,7 +150,6 @@ class http_class {
         
         $url = split (':',$url,2);
         $transport_type = $url[0];
-        $url = $url[1];
         switch($transport_type) {
                 case 'http':
                     $transport_type = 'tcp://';
@@ -126,19 +158,39 @@ class http_class {
                     $transport_type = 'tls://';
                     break;
             }
-        
-            $url = split("/",preg_replace("#^/{1,}#",'',$url),2);
-            $url = $url[0];
-            $url = split(":",$url);
-            $port = array_key_exists(1,$url) ? $url[1] : $this->default_port;
-            $url = $url[0];
-        
-            $this->connection = fsockopen($transport_type.$url, $port, $errno, $errstr, $this->timeout);
-        
-            if (!$this->connection) 
-                return sprintf ('%s (%s)', $errstr, $errno);
+        $url = $url[1];
+        $url = split("/",preg_replace("#^/{1,}#",'',$url),2);
+        $url = $url[0];
+        $url = split(":",$url);
+        $this->default_port = "toto";
+        $port = array_slice($url,-1);
+        $port = $port[0];
+        (preg_match("#[0-9]{2,}#",$port)) ? $port=array_pop($url) : $port=$this->default_port;
+        $url = implode(":",$url);
+        $error = sprintf(_("Cannot resolve url: %s"),$url);
+        $ip = gethostbyname($url);
+        $ip = @gethostbyaddr($ip);
+        if (!$ip)
+            if ($this->with_exceptions)
+                throw new httpException($error);
+            else
+                trigger_error($error,E_USER_WARNING);
+        if (strstr($url,":")) // we got an ipv6 address
+            if (!strstr($url,"[")) // it is not escaped
+                $url = sprintf("[%s]",$url);
+        $this->connection = @fsockopen($transport_type.$url, $port, $errno, $errstr, $this->timeout);
+        $error = sprintf (_('Unable to connect to "%s%s port %s": %s'),
+            $transport_type,
+            $url,
+            $port,
+            $errstr);
 
-            $this->connected = 1;
+        if (!$this->connection)
+            if ($this->with_exceptions)
+                throw new httpException($error,$errno);
+            else
+                trigger_error($error,E_USER_WARNING);
+        $this->connected = 1;
     }
     // }}}
 
@@ -225,7 +277,7 @@ class http_class {
         //$read = array($this->connection);
         $status = stream_get_meta_data($this->connection);
         //if(stream_select($read, $write = NULL, $except = NULL, 0) === 1) {
-        if ($status['unread-bytes']) {
+        if (isset($status['unread-bytes']) && $status['unread-bytes']) {
             // server talks!
             trigger_error(_("http_class: server talk first, quit"),E_USER_WARNING);
             return false;
@@ -242,7 +294,7 @@ class http_class {
         
         //if(stream_select($read, $write = NULL, $except = NULL, 0) === 1) // server talks!
         $status = stream_get_meta_data($this->connection);
-        if ($status['unread-bytes']) 
+        if (isset($status['unread-bytes']) && $status['unread-bytes'])
              return "server-talk";
         
         if (!$error) {
@@ -283,7 +335,7 @@ class http_class {
             $this->request_headers[$header] = $value;
             //if(stream_select($read, $write = NULL, $except = NULL, 0) === 1)
             $status = stream_get_meta_data($this->connection);
-            if ($status['unread-bytes'])
+            if (isset($status['unread-bytes']) && $status['unread-bytes'])
                 return "server-talk";
                 
             if (!$error) {
@@ -298,7 +350,7 @@ class http_class {
             //if($strselect = stream_select($read, $write = NULL, $except = NULL, 0,$this->data_timeout*1000) === 1) 
             usleep($this->data_timeout*1000);
             $status = stream_get_meta_data($this->connection);
-            if ($status['unread-bytes'])
+            if (isset($status['unread-bytes']) && $status['unread-bytes'])
                 return "server-talk";
 
             if (!$error) {
@@ -318,7 +370,7 @@ class http_class {
                     //if(stream_select($read, $write = NULL, $except = NULL, 0) === 1)
                     usleep($this->data_timeout*1000);
                     $status = stream_get_meta_data($this->connection);
-                    if ($status['unread-bytes'])
+                    if (isset($status['unread-bytes']) && $status['unread-bytes'])
                         return "server-talk";
                 
                     // not very clean...
@@ -344,7 +396,7 @@ class http_class {
 
                         //if(stream_select($read, $write = NULL, $except = NULL, 0) === 1)
                         $status = stream_get_meta_data($this->connection);
-                        if ($status['unread-bytes'])
+                        if (isset($status['unread-bytes']) && $status['unread-bytes'])
                             return "server-talks";
                 
                         // not very clean...
@@ -358,7 +410,7 @@ class http_class {
                 
                 //if(stream_select($read, $write = NULL, $except = NULL, 0) === 1)
                  $status = stream_get_meta_data($this->connection);
-                 if ($status['unread-bytes'])
+                 if (isset($status['unread-bytes']) && $status['unread-bytes'])
                     return "server-talks";
                 
             }
