@@ -1,8 +1,20 @@
 <?php
+
+$debug = 0;
+error_reporting(E_ALL|E_STRICT);
 /* printing an utf-8 file */
 require_once('PrintIPP.php'); 
 $ipp = new PrintIPP(); 
-$ipp->setHost("localhost"); 
+$ipp->with_exceptions = 1;
+
+//
+$ipp->setHost("localhost");
+  // various tests for ipv6 and SSL you can enable instead
+	//$ipp->setHost("ip6-localhost");
+	//$ipp->setHost("127.0.0.1");
+	//$ipp->setHost("::1");
+	//$ipp->ssl = 1;
+	$ipp->setPort("631");
 $ipp->setPrinterURI("/printers/Parallel_Port_1"); 
 $ipp->debug_level = 3; // Debugging very verbose
 $ipp->setLog('/tmp/printipp','file',3); // logging very verbose
@@ -13,9 +25,18 @@ $ipp->setMimeMediaType('text/plain');
 $ipp->setAttribute('number-up',1); // pages per sheet
 $ipp->setSides(1); // by default: 2 = two-sided-long-edge // other choices: 1 = one-sided // 2CE = two-sided-short-edge
 $ipp->setData("./testfiles/test-utf8.txt");//Path to file.
-printf(_("Job status: %s"), $ipp->printJob()); // Print job, display job status 
-$ipp->CancelJob($ipp->last_job); // uncomment to deforrest ;)
-$ipp->printDebug(); // display debugging output
+try
+{
+  printf(_("Job status: %s"), $ipp->printJob()); // Print job, display job status 
+  $ipp->CancelJob($ipp->last_job); // uncomment to deforrest ;)
+}
+catch (httpException $e)
+{
+  printf("%s\nerrno: %s\n",$e->getMessage(),$e->getErrno());
+  trigger_error("I prefer to quit", E_USER_ERROR);
+}
+if ($debug)
+  $ipp->printDebug(); // display debugging output
 ?>
 
 END "PRINT THEN CANCEL" TEST
