@@ -1,9 +1,9 @@
 <?php
 /* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
- /* @(#) $Header: /sources/phpprintipp/phpprintipp/tiny/PrintIPP.php,v 1.2 2006/01/12 21:12:11 harding Exp $
+ /* @(#) $Header: /sources/phpprintipp/phpprintipp/tiny/PrintIPP.php,v 1.4 2008/05/12 03:59:38 harding Exp $
  *
- * Version: 0.3 
- * Class PrintIPP - Send Basic IPP requests, Get IPP Responses.
+ * Version: 0.4 
+ * Class PrintIPP (Tiny) - Send Basic IPP requests, Get IPP Responses.
  *
  *   Copyright (C) 2005-2006  Thomas HARDING
  *   Parts Copyright (C) 2005-2006 Manuel Lemos
@@ -41,7 +41,7 @@
 
 /***********************
 *
-* class PrintIPP 
+* class PrintIPP (Tiny)
 *
 ************************/
 
@@ -62,8 +62,12 @@ class PrintIPP {
     public $debug_level = 2; // max 3: very verbose
     public $available_printers = array();
     public $printers_uri = array();
+    public $with_exceptions = 0;
     public $response;
-    
+    public $http_timeout = 0; // timeout at http connection (seconds) 0 => default => 30.
+    public $http_data_timeout = 0; // data reading timeout (milliseconds) 0 => default => 30.
+    public $ssl = false;
+
     // {{{ protected variables;
     
     protected $log_level = 2; // max 3: very verbose
@@ -101,6 +105,17 @@ class PrintIPP {
     
     // {{{ constructor
     public function __construct() {
+        $tz = getenv("date.timezone");
+        if (!$tz)
+            $tz = date_default_timezone_get();
+        date_default_timezone_set($tz);
+
+        $this->meta = new stdClass();
+        $this->setup = new stdClass();
+        $this->values = new stdClass();
+        $this->serveroutput = new stdClass();
+        $this->error_generation = new stdClass();
+        $this->_parsing = new stdClass();
         self::_initTags ();
     }
     // }}}
@@ -140,7 +155,6 @@ class PrintIPP {
         while (strlen($length) < 2)
             $length = chr(0x00) . $length;
         
-
         $this->meta->printer_uri = chr(0x45) // uri type | value-tag
                          .  chr(0x00) . chr(0x0B) // name-length
                          .  "printer-uri" // printer-uri | name
@@ -151,6 +165,7 @@ class PrintIPP {
         
         self::_putDebug(sprintf(_("Printer URI: %s\n"),$uri));
         
+
         $this->setup->uri = 1;
 
     }
@@ -393,8 +408,8 @@ class PrintIPP {
     }
     // }}}
 
-    // {{{ setAuthentification ($username,$password)
-    public function setAuthentification ($username,$password) {
+    // {{{ setAuthentication ($username,$password)
+    public function setAuthentication ($username,$password) {
         $this->password = $password;
         $this->username = $username;
         
@@ -403,6 +418,12 @@ class PrintIPP {
     }
     // }}}
 
+    // {{{ setAuthentification ($username,$password)
+    public function setAuthentification ($username,$password) {
+        self::setAuthentication ($username,$password);
+    }
+    // }}}
+
     // {{{ setSides ($sides=2)
     public function setSides ($sides=2) {
         
@@ -730,6 +751,8 @@ class PrintIPP {
     /*
         This function Copyright (C) 2005-2006 Thomas Harding, Manuel Lemos  
     */
+            
+            $this->response_completed[] = "no";       
             unset($this->serverouptut);
 
             self::_putDebug( _("Processing HTTP request\n\n"));
@@ -738,6 +761,8 @@ class PrintIPP {
             $this->serveroutput->body = "";
             
             $http = new http_class;
+            $http->with_exceptions = $this->with_exceptions;
+            
             
             if ($this->debug_http) {
                 $http->debug = 1;
@@ -748,18 +773,18 @@ class PrintIPP {
             }
             
             $url="http://".$this->host.":".$this->port;
-            if ($this->tls == 1)
+            if ($this->ssl)
                 $url="https://".$this->host.":".$this->port;
             
-            $http->timeout=0;
-            $http->data_timeout=0;
+            
+            $http->timeout = $this->http_timeout;
+            $http->data_timeout= $this->http_data_timeout;
             
             $http->force_multipart_form_post = false;
             $http->user = $this->username;
             $http->password = $this->password;
             
             $error=$http->GetRequestArguments($url,$arguments);
-
             $arguments["RequestMethod"]="POST";
             $arguments["Headers"] = array("Content-Type" => "application/ipp");
             $arguments["BodyStream"] = array( array("Data" => $post_values["Data"]) );
@@ -772,17 +797,14 @@ class PrintIPP {
             
             $arguments["RequestURI"] = $uri;
             
-            
-            
             $error=$http->Open($arguments);
             
-            
+            //echo "<pre>";print_r(memory_get_usage());echo "<pre>";
+            // ~1.5M
+
             if($error=="") {
                 $error=$http->SendRequest($arguments);
                 
-            //echo "<pre>";print_r(memory_get_usage());echo "<pre>";
-            // ~500k
-
                 if($error=="") {
                 
                     self::_putDebug( "\nH T T P    R E Q U E S T :\n\n");
@@ -1541,7 +1563,7 @@ class PrintIPP {
 
 /*************************
 *
-* Embedded HTTP client (Version: 0.2 EXPERIMENTAL)
+* Embedded HTTP client
 *
 **************************/
 
@@ -1559,9 +1581,9 @@ class http_class {
     public $request_body = "Not a useful information";
     public $status;
     public $window_size = 1400; // chunk size of data
-    
+    public $with_exceptions = 0; // compatibility mode for old scripts
+
     private $default_port = 631;
-    
     private $headers;
     private $reply_headers = array();
     private $reply_body = array();
@@ -1570,6 +1592,8 @@ class http_class {
     private $bodystream = array();
     private $last_limit;
     private $connected;
+    private $nc = 1;
+    private $user_agent = "PrintIPP http backend V0.3";
     // }}}
             
     // {{{ constructor
@@ -1592,7 +1616,7 @@ class http_class {
         $arguments["URL"] = $this->arguments["URL"] = $url;
         $arguments["RequestMethod"] = $this->arguments["RequestMethod"] = "POST";
         
-        $this->headers["User-agent"] = "PrintIPP Tiny version 0.1";
+        $this->headers["User-agent"] = $this->user_agent;
         $this->headers["Content-Type"] = "application/octet-stream";
     
     }
@@ -1601,13 +1625,6 @@ class http_class {
     // {{{ Open ($arguments)
     public function Open ($arguments) {
         
-        /*
-        echo "<pre>";
-        print_r(stream_get_transports());
-        print_r(stream_get_wrappers());
-        echo "</pre>";
-        */
-
         if (!$this->timeout)
             $this->timeout = 30;
         $url = $arguments["URL"];
@@ -1615,7 +1632,6 @@ class http_class {
         
         $url = split (':',$url,2);
         $transport_type = $url[0];
-        $url = $url[1];
         switch($transport_type) {
                 case 'http':
                     $transport_type = 'tcp://';
@@ -1624,20 +1640,39 @@ class http_class {
                     $transport_type = 'tls://';
                     break;
             }
-        
-            $url = split("/",preg_replace("#^/{1,}#",'',$url),2);
-            $url = $url[0];
-            $url = split(":",$url);
-            $port = array_key_exists(1,$url) ? $url[1] : $this->default_port;
-            $url = $url[0];
-        
-            //echo "$transport_type#$url#$port";flush();
-            $this->connection = fsockopen($transport_type.$url, $port, $errno, $errstr, $this->timeout);
-        
-            if (!$this->connection) 
-                return sprintf ('%s (%s)', $errstr, $errno);
-
-            $this->connected = 1;
+        $url = $url[1];
+        $url = split("/",preg_replace("#^/{1,}#",'',$url),2);
+        $url = $url[0];
+        $url = split(":",$url);
+        $this->default_port = "toto";
+        $port = array_slice($url,-1);
+        $port = $port[0];
+        (preg_match("#[0-9]{2,}#",$port)) ? $port=array_pop($url) : $port=$this->default_port;
+        $url = implode(":",$url);
+        $error = sprintf(_("http_class: Cannot resolve url: %s"),$url);
+        $ip = gethostbyname($url);
+        $ip = @gethostbyaddr($ip);
+        if (!$ip)
+            if ($this->with_exceptions)
+                throw new Exception($error);
+            else
+                trigger_error($error,E_USER_WARNING);
+        if (strstr($url,":")) // we got an ipv6 address
+            if (!strstr($url,"[")) // it is not escaped
+                $url = sprintf("[%s]",$url);
+        $this->connection = fsockopen($transport_type.$url, $port, $errno, $errstr, $this->timeout);
+        $error = sprintf (_('http_class: unable to connect to "%s%s port %s": "%s" (errno %s)'),
+            $transport_type,
+            $url,
+            $port,
+            $errstr,
+            $errno);
+        if (!$this->connection)
+            if ($this->with_exceptions)
+                throw new Exception($error);
+            else
+                trigger_error($error,E_USER_WARNING);
+        $this->connected = 1;
     }
     // }}}
 
@@ -1661,14 +1696,18 @@ class http_class {
             return("SendRequest: need authentication but no mechanism provided");
 
          $authtype = split(' ',$this->reply_headers["www-authenticate"]);
-         
-         switch ($authtype[0]) {
+         $authtype = strtolower($authtype[0]);
+
+         switch ($authtype) {
             case 'basic':
                 $pass = base64_encode($this->user.":".$this->password);
                 $arguments["Headers"]["Authorization"] = "Basic ".$pass;
                 break;
+            case 'digest':
+                $arguments["Headers"]["Authorization"] = self::_BuildDigest();
+                break;
             default:
-                return sprintf(_("SendRequest: need '%s' authentication mechanism, but have not, sorry"),$authtype[0]);
+                return sprintf(_("http_class: need '%s' authentication mechanism, but have not, sorry"),$authtype[0]);
          }
         
          self::Close();
@@ -1707,7 +1746,7 @@ class http_class {
 *
 **********************/
 
-    // {{{ StreamRequest ($arguments)
+    // {{{ _StreamRequest ($arguments)
     private function _StreamRequest ($arguments) {
         
         $this->status = "";
@@ -1717,8 +1756,10 @@ class http_class {
         $this->arguments = $arguments;
         $this->arguments["Headers"] = array_merge($this->headers,$this->arguments["Headers"]);
         
-        $read = array($this->connection);
-        if(stream_select($read, $write = NULL, $except = NULL, 0) === 1) {
+        //$read = array($this->connection);
+        $status = stream_get_meta_data($this->connection);
+        //if(stream_select($read, $write = NULL, $except = NULL, 0) === 1) {
+        if (isset($status['unread-bytes']) && $status['unread-bytes']) {
             // server talks!
             trigger_error(_("http_class: server talk first, quit"),E_USER_WARNING);
             return false;
@@ -1732,8 +1773,11 @@ class http_class {
         $string = sprintf("POST %s HTTP/1.1\r\n",$this->arguments["RequestURI"]);
         $error = fwrite($this->connection,$string);
         $this->request_headers[$string] = '';
-        if(stream_select($read, $write = NULL, $except = NULL, 0) === 1) // server talks!
-            return "server-talk";
+        
+        //if(stream_select($read, $write = NULL, $except = NULL, 0) === 1) // server talks!
+        $status = stream_get_meta_data($this->connection);
+        if (isset($status['unread-bytes']) && $status['unread-bytes'])
+             return "server-talk";
         
         if (!$error) {
             trigger_error(_("Error while puts first header"),E_USER_WARNING);
@@ -1769,9 +1813,11 @@ class http_class {
         $this->arguments["Headers"]["Content-Length"] =  $content_length;
         foreach ($this->arguments["Headers"] as $header => $value) {
             
-            $error = fwrite($this->connection,sprintf("%s: %s\r\n", $header, $value));
+            $error = @fwrite($this->connection,sprintf("%s: %s\r\n", $header, $value));
             $this->request_headers[$header] = $value;
-            if(stream_select($read, $write = NULL, $except = NULL, 0) === 1)
+            //if(stream_select($read, $write = NULL, $except = NULL, 0) === 1)
+            $status = stream_get_meta_data($this->connection);
+            if (isset($status['unread-bytes']) && $status['unread-bytes'])
                 return "server-talk";
                 
             if (!$error) {
@@ -1782,10 +1828,12 @@ class http_class {
         
             $error = fwrite($this->connection,"\r\n");
             fflush($this->connection);
-
-            if($strselect = stream_select($read, $write = NULL, $except = NULL, 0,$this->data_timeout*1000) === 1) 
+            
+            //if($strselect = stream_select($read, $write = NULL, $except = NULL, 0,$this->data_timeout*1000) === 1) 
+            usleep($this->data_timeout*1000);
+            $status = stream_get_meta_data($this->connection);
+            if (isset($status['unread-bytes']) && $status['unread-bytes'])
                 return "server-talk";
-                //echo "strselect".$strselect."<br />";
 
             if (!$error) {
                 trigger_error(_("Error while ends HTTP headers"),E_USER_WARNING);
@@ -1801,7 +1849,10 @@ class http_class {
             if ($type == "Data") {
                 $streamed_length = 0;
                 while ($streamed_length < strlen($value)) {
-                    if(stream_select($read, $write = NULL, $except = NULL, 0) === 1)
+                    //if(stream_select($read, $write = NULL, $except = NULL, 0) === 1)
+                    usleep($this->data_timeout*1000);
+                    $status = stream_get_meta_data($this->connection);
+                    if (isset($status['unread-bytes']) && $status['unread-bytes'])
                         return "server-talk";
                 
                     // not very clean...
@@ -1825,7 +1876,9 @@ class http_class {
                             return _("cannot read file to upload");
                             }
 
-                        if(stream_select($read, $write = NULL, $except = NULL, 0) === 1)
+                        //if(stream_select($read, $write = NULL, $except = NULL, 0) === 1)
+                        $status = stream_get_meta_data($this->connection);
+                        if (isset($status['unread-bytes']) && $status['unread-bytes'])
                             return "server-talks";
                 
                         // not very clean...
@@ -1837,7 +1890,9 @@ class http_class {
                     }
                 }
                 
-                if(stream_select($read, $write = NULL, $except = NULL, 0) === 1)
+                //if(stream_select($read, $write = NULL, $except = NULL, 0) === 1)
+                 $status = stream_get_meta_data($this->connection);
+                 if (isset($status['unread-bytes']) && $status['unread-bytes'])
                     return "server-talks";
                 
             }
@@ -1846,7 +1901,7 @@ class http_class {
     }
     // }}}
     
-    // {{{ ReadReply ()
+    // {{{ _ReadReply ()
     private function _ReadReply () {
       
         $this->reply_headers = array();
@@ -1870,7 +1925,12 @@ class http_class {
         $this->status = strtolower($headers[0]);
       
         foreach($headers as $header) {
-            $header = preg_split("#: #",strtolower($header));
+            if (!preg_match('#www-authenticate: #i',$header))
+                $header = strtolower($header);
+            
+            $header = preg_split("#: #",$header);
+            $header[0] =  strtolower($header[0]);
+
                 $this->reply_headers["{$header[0]}"] = array_key_exists(1,$header) ? $header[1] : "";
             }
             unset ($this->reply_headers['']);
@@ -1880,8 +1940,11 @@ class http_class {
         for ($i = 0 ; $i < 2 ; $i++) {
             if (self::_ReadStream() === "completed")
                 break;
-            $strselect = stream_select($read, $write = NULL, $except = NULL, 0,$this->data_timeout*1000);
-            if (!$strselect)
+            //$strselect = stream_select($read, $write = NULL, $except = NULL, 0,$this->data_timeout*1000);
+            //if (!$strselect)
+            usleep($this->data_timeout*1000);
+            $status = stream_get_meta_data ($this->connection);
+            if ($status["unread_bytes"] == 0)
                 break;
             }
             
@@ -1895,26 +1958,122 @@ class http_class {
         $content_length = 0;
         if (array_key_exists("content-length",$this->reply_headers));
             $content_length = $this->reply_headers["content-length"];
-            
+
         stream_set_blocking ($this->connection, 0 );
         usleep($this->data_timeout * 1000);
+        $total = 0;
+        $chunk = true;
         while (true) {
-            usleep (1000);
-            $chunk = fread($this->connection,$this->window_size);
-            //echo $content_length."<br />";
             if ($content_length)
                 if (strlen($this->reply_body) >= $content_length)
                     return "completed";
             else
                 if (!$chunk)
                     break;
-
+            usleep (1000);
+            $chunk = @fread($this->connection,$this->window_size);
             $this->reply_body .= $chunk;
+            
+            $status = stream_get_meta_data ($this->connection);
+            if ($status["unread_bytes"] == 0)
+                break;
             }
-
+        stream_set_blocking ($this->connection, 1 );
     return true;
     }
     // }}}
+
+    // {{{ _BuildDigest ()
+    private function _BuildDigest () {
+        
+        $auth = $this->reply_headers["www-authenticate"];
+        
+        list($head, $auth) = split(" ",$auth,2);
+
+        $auth=split(", ",$auth);
+        foreach ($auth as $sheme) {
+            list($sheme,$value) = split('=',$sheme);
+            $fields[$sheme] = trim(trim($value),'"');
+        }
+
+        $nc = sprintf('%x',$this->nc);
+        $prepend = "";
+        while ((strlen($nc) + strlen($prepend)) < 8)
+            $prepend .= "0";
+        $nc=$prepend.$nc;
+        
+        $cnonce = "printipp";
+        
+        $username = $this->user;
+        $password = $this->password;
+    
+        $A1 = $username.":".$fields["realm"].":".$password;
+        
+        if (array_key_exists("algorithm",$fields)) {
+            $algorithm = strtolower($fields["algorithm"]);
+            switch ($algorithm) {
+                case "md5":
+                    break;
+                case "md5-sess":
+                    $A1 = $username.":".$fields["realm"].":".$password.":".$fields['nonce'].":".$cnonce;
+                    break;
+                case "token":
+                    trigger_error("http_class: digest Authorization: algorithm 'token' not implemented", E_USER_WARNING);
+                    return false;
+                    break;
+                }
+
+            }
+        
+        $A2 = "POST:".$this->arguments["RequestURI"];
+
+        
+        if (array_key_exists("qop",$fields)) {
+            $qop = strtolower($fields["qop"]);
+            $qop = split(" ",$qop);
+            if (in_array("auth",$qop))
+                $qop = "auth";
+            else {
+                trigger_error("http_class: digest Authorization: qop others than 'auth' not implemented", E_USER_WARNING);
+                return false;
+                }
+            }
+
+            
+        $response = md5(md5($A1).":". $fields["nonce"].":" .md5($A2));
+        
+        if (isset($qop) && ($qop == "auth"))
+        $response = md5(md5($A1).":".$fields["nonce"].":".$nc.":".$cnonce.":".$qop.":".$A2);
+
+        
+        $auth_scheme = sprintf('Digest username="%s", realm="%s", nonce="%s", uri="%s", response="%s"',
+                                $username,
+                                $fields["realm"],
+                                $fields['nonce'],
+                                $this->arguments["RequestURI"],
+                                $response
+                                );
+        
+        if (isset($algorithm))
+            $auth_scheme .= sprintf(', algorithm="%s"',$algorithm);
+
+        if (isset($qop))
+            $auth_scheme .= sprintf(', cnonce="%s"',$cnonce);
+        
+        if(array_key_exists("opaque",$fields))
+            $auth_scheme .= sprintf(', opaque="%s"',$fields['opaque']);
+      
+        if (isset($qop))
+            $auth_scheme .= sprintf(', qop="%s"',$qop);
+      
+        $auth_scheme .= sprintf(', nc=%s',$nc);
+      
+        $this->nc++;
+        
+    return $auth_scheme;
+    }
+    // }}}
+
 };
 
 
